/**
 * Copyright (C) 2011 Agrosense <info@cloudfarming.nl>
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package nl.cloudfarming.client.stock;

import java.util.Date;
import java.util.List;
import javax.persistence.Query;
import nl.cloudfarming.client.model.CommonModelService;
import nl.cloudfarming.client.model.ConcentrationDosageUnit;
import nl.cloudfarming.client.model.CropProtection;
import nl.cloudfarming.client.model.DosageUnit;
import nl.cloudfarming.client.model.crop.Crop;
import org.openide.util.lookup.ServiceProvider;

/**
 * Class for common CropProtection service methods
 * Always use serviceProvider to instantiate
 * @author Merijn Zengers
 */
@ServiceProvider(service = StockService.class)
public class StockServiceImpl extends CommonModelService implements StockService {

    /**
     * Find the CropProtection based on it's authorization number 
     * @param authorizationNumber the authorization number to look for
     * @return CropProtection if found
     */
    @Override
    public final List<CropProtection> findCropProtectionByAuthorizationNumberAndApplicationMethod(int authorizationNumber, String applicationMethod) {
        try {
            startTransaction();
            Query query = getEntityManager().createNamedQuery(CropProtection.Q_FIND_CROPPROTECTION_BY_AUTHORIZATION_NUMBER);
            query.setParameter("authorizationNumber", authorizationNumber);
            query.setParameter("applicationMethod", applicationMethod);
            return query.getResultList();
        } catch (javax.persistence.NoResultException ex) {
            return null;
        }
    }

    /**
     * Close a CropProtection.
     * Fills the endDate with today
     * @param cropProtection
     * @return returns true if datUntil was null false if dateUntil was already set.
     */
    @Override
    public boolean endCropProtection(CropProtection cropProtection) {
        if (cropProtection.getDateUntil() == null) {
            startTransaction();
            cropProtection.setDateUntil(new Date());
            getEntityManager().merge(cropProtection);
            getEntityManager().getTransaction().commit();
            return true;
        }
        return false;
    }

    /**
     * Persist the CropProtection in transaction
     * @param cropProtection The CropProtection to persist in transaction
     */
    @Override
    public void persistInTransaction(CropProtection cropProtection) {
        startTransaction();
        getEntityManager().persist(cropProtection);
        getEntityManager().getTransaction().commit();
    }

    /**
     * Remove persisted CropProtections
     * @param cropProtection the CropProtection to persist
     */
    @Override
    public void removeCropProtection(CropProtection cropProtection) {
        startTransaction();
        getEntityManager().remove(cropProtection);
        getEntityManager().getTransaction().commit();
    }

    /**
     * Find all the Persisted CropProtections
     * @return a List of found CropProtections
     */
    @Override
    public List<CropProtection> findAllCropProtections() {
        startTransaction();
        return getEntityManager().createNamedQuery(CropProtection.Q_FIND_ALL_CROPPROTECTION).getResultList();
    }

    /**
     * Find all the CropProtection object for a certain crop
     * @param crop the crop to find the CropProtection for
     * @return all the CropProtections found
     */
    @Override
    public List<CropProtection> getCropProtectionByCrop(Crop crop) {
        startTransaction();
        Query query = getEntityManager().createNamedQuery(CropProtection.Q_FIND_CROPPROTECTIONS_BY_CROP_KEY);
        query.setParameter("cropKey", crop.getBundleKey());
        List<CropProtection> cropProtections = query.getResultList();
        
        //Set the crop to the crop known in our application
        for (CropProtection cropProtection : cropProtections) {
            cropProtection.setCrop(crop);
            //Also set the dossage units
            //TODO: Get these from the webservice
            cropProtection.setDosageUnit(DosageUnit.UNKNOWN);
            cropProtection.setConcentrationDosageUnit(ConcentrationDosageUnit.UNKNOWN);
        }

        return cropProtections;
    }

    @Override
    protected String getModuleName() {
        return StockModule.MODULE_NAME;
    }

    @Override
    protected void initExplorer() {
    }
}
