/**
 * Copyright (C) 2011 Agrosense <info@cloudfarming.nl>
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a copy
 * of the License at
 *
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package nl.cloudfarming.client.stock.scheduler;

import java.util.Collection;
import java.util.Date;
import java.util.List;
import java.util.TimerTask;
import nl.cloudfarming.client.model.CropProtection;
import nl.cloudfarming.client.stock.CropProtectionService;
import nl.cloudfarming.client.stock.StockService;
import nl.cloudfarming.client.stock.offline.OfflineCropProtectionFactory;
import org.openide.util.Lookup;

/**
 * Worker class to update the persisted CropProtection or create new ones from
 * data acquired from the web services
 *
 * @author Merijn Zengers
 */
public class WebServiceUpdater extends TimerTask {

    //Use getStockService() to get reference to this object
    private StockService stockService;

    public WebServiceUpdater(){
        stockService = Lookup.getDefault().lookup(StockService.class);
    }
    
    /**
     * Only for test purpose
     * @param stockService 
     */
    protected void setStockService(StockService stockService) {
        this.stockService = stockService;
    }


    /**
     * Will lookup all the the ServiceProviders of type CropProtectionService.class
     * Will get all the cropProtections they provide through getCropProtection()
     * Will create or update existing CropProtecton with new values
     */
    @Override
    public void run() {
        Collection<? extends CropProtectionService> cropProtectionServices = Lookup.getDefault().lookupAll(CropProtectionService.class);
        for (CropProtectionService service : cropProtectionServices) {
            for (CropProtection cropProtection : service.getCropProtections()) {
                updateCropProtection(cropProtection);
            }
        }
        if(stockService.findAllCropProtections().isEmpty()){
            OfflineCropProtectionFactory factory = new OfflineCropProtectionFactory();
            for(CropProtection cropProtection :factory.getOfflineCropProtections()){
                stockService.persistInTransaction(cropProtection);
            }
        }
    }

    /**
     * Will update the persisted CropProtections
     * Will search the persisted CropProtection based on it's authorization Number.
     * If found will check if found is equal to supplied CropProtection
     * If not equal will
     * - fill endDate of persisted CropProtection with now
     * - fill startDate of supplied CropProtection with now
     * - persist the supplied CropProtection
     * If not found will
     * - fill startDate of supplied CropProtection with now
     * - persist the supplied CropProtection
     * @param cropProtection The CropProtecion to update or persist
     */
    protected void updateCropProtection(CropProtection cropProtection) {
        List<CropProtection> foundCropProtections = stockService.findCropProtectionByAuthorizationNumberAndApplicationMethod(cropProtection.getAuthorizationNumber(), cropProtection.getApplicationMethod());
        if (foundCropProtections != null && !foundCropProtections.isEmpty()) {
            for (CropProtection found : foundCropProtections) {
                if (found.getDateUntil() == null) {
                    if (!cropProtectionIsequal(found, cropProtection)) {
                        stockService.endCropProtection(found);
                        cropProtection.setDateFrom(new Date());
                        stockService.persistInTransaction(cropProtection);
                    }
                }
            }
        } else {
            cropProtection.setDateFrom(new Date());
            stockService.persistInTransaction(cropProtection);
        }
    }
    
    protected boolean cropProtectionIsequal(CropProtection obj, CropProtection other){
        if (obj == null) {
            return false;
        }
        if (other.getClass() != obj.getClass()) {
            return false;
        }
        if (obj.getAuthorizationNumber() != other.getAuthorizationNumber()) {
            return false;
        }
        if (obj.getAgent() != other.getAgent() && (obj.getAgent() == null || !obj.getAgent().equals(other.getAgent()))) {
            return false;
        }
        if (obj.getCrop() != other.getCrop() && (obj.getCrop() == null || !obj.getCrop().equals(other.getCrop()))) {
            return false;
        }
        if ((obj.getApplicationMethod() == null) ? (other.getApplicationMethod() != null) : !obj.getApplicationMethod().equals(other.getApplicationMethod())) {
            return false;
        }
        if (obj.getSafetyTerm() != other.getSafetyTerm()) {
            return false;
        }
        if (obj.getEntryDelay() != other.getEntryDelay()) {
            return false;
        }
        if (obj.getHarvestDelay() != other.getHarvestDelay()) {
            return false;
        }
        if (obj.getEnvironmentalIssues() != other.getEnvironmentalIssues() && (obj.getEnvironmentalIssues() == null || !obj.getEnvironmentalIssues().equals(other.getEnvironmentalIssues()))) {
            return false;
        }
        if (obj.getDosageUnit() != other.getDosageUnit()) {
            return false;
        }
        if (obj.getConcentrationDosageUnit() != other.getConcentrationDosageUnit()) {
            return false;
        }
        if (obj.getDateFrom() != other.getDateFrom() && (obj.getDateFrom() == null || !obj.getDateFrom().equals(other.getDateFrom()))) {
            return false;
        }
        if (obj.getDateUntil() != other.getDateUntil() && (obj.getDateUntil() == null || !obj.getDateUntil().equals(other.getDateUntil()))) {
            return false;
        }
        return true;
    }
    
    
}
